<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Services\AprovadorService;

class AprovadorController
{

    public function __construct(AprovadorService $service)
    {
        $this->service = $service;
        $message       = 0;
    }
    
    public function saveAprovador(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            
            $row['VRAPROVCMP'] = self::transformValueMaskToFloat($row['VRAPROVCMP']);
            
            if (isset($row['__is_new']) && $row['__is_new']) {
                $this->service->saveAprovador($row);
            } else {
                $this->service->updateAprovador($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function saveMultiploAprovador(DTO\Request\Row $request, DTO\Response $response)
    {
        $rows = $request->getRow();
        
        foreach ($rows as $row) {
            $row['VRAPROVCMP'] = self::transformValueMaskToFloat($row['VRAPROVCMP']);
    
            $this->service->saveAprovador($row);
        }
    }
    
    public function deleteAprovador(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
            
        $aprovador = $this->service->findAprovador($row);
        
        if ($aprovador['APROVACAO'] == 0) {
            $this->service->deleteTodosAprovadoresRelacionadosAoAprovador($row);
            $this->service->deleteAprovador($row);    
        } else {
            $response->setError(new Error("OPERATION_BLOCKED_DEL_PENDING_APPROVALS", 500));
        }
    }
    
    public function saveAprovadorRelacionado(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $this->service->saveAprovadorRelacionado($row);
    }
    
    public function deleteAprovadorRelacionado(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $this->service->deleteAprovadorRelacionado($row);
    }
    
    public function importaGestoresAuto($filial, $nrAlcada)
    {
        $alcada   = $this->service->findAlcada($nrAlcada);
        $gestores = $this->service->findGestores($filial);
        $messageParam = 0;
        $temGestorRepetido = false;
        
        foreach ($gestores as $key=>$gestor) {
            $columnGestorAprovacao = self::getGestorAprovacao($key);
            
            if (!empty($alcada[$columnGestorAprovacao])) {
                if ($gestor != null) {
                    try {
                        $this->service->saveAprovador(
                            array(
                                'NRCONFIGALCADA' => $alcada['NRCONFIGALCADA'],
                                'CDFILIAL'       => $filial,
                                'CDOPERADOR'     => $gestor,
                                'VRAPROVCMP'     => $alcada[$columnGestorAprovacao],
                                'IDEXIGJUSTIFICATIVA' => 'N',
                            )
                        );
                    } catch(\Exception $e) {
                        if (preg_match('/UN_ALCADAGERFILI/', $e->getMessage()) === 1) {
                            $temGestorRepetido = true;
                        } else {
                            throw $e;
                        }
                    }
                } else {
                    $messageParam++;
                }
            }
        }
        
        return self::defineMessageWithParam($messageParam, $temGestorRepetido);
    }
    
    public function getGestorAprovacao($index)
    {
        switch ($index) {
            case 'DIRETORIA':
                return 'VRDIRETORIA';
            case 'GERENTE':
                return 'VRGERREGIONAL';
            case 'SUPERVISOR':
                return 'VRSUPERVISOR';
            case 'PLANEJAMENTO':
                return 'VRPLANEJAMENTO';
        }
    }
    
    public function defineMessageWithParam($param, $temGestorRepetido)
    {
        switch ($param) {
            case 0:
                return 'Gestores incluídos com sucesso.';
            case 4:
                return 'Não existem gestores cadastrados para a unidade.';
            default:
                $mensagemRetorno = 'Nem todos os gestores estão cadastrados para a unidade.';
                
                if ($temGestorRepetido) {
                    $mensagemRetorno .= '|Gestores repetidos não foram incluídos.';
                }
                
                return $mensagemRetorno;
        }
    }
    
    public function transformValueMaskToFloat($value)
    {
        $arrayReplace = array(
            'R$',
            '.',
        );
        $value = str_replace($arrayReplace, '', $value);
        $value = str_replace(',', '.', $value);
        
        return $value;
    }
}
